#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Home Broker API - Market data downloader
# https://github.com/crapher/pyhomebroker.git
#
# Copyright 2020 Diego Degese
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

brokers = [
    {
    'broker_id': 12,
    'name': 'Buenos Aires Valores S.A.',
    'page': ':operarhb.bavsa.com'
    },
    {
    'broker_id': 20,
    'name': 'Proficio Investment S.A.',
    'page': 'newsystem.proficioinvestment.com.ar'
    },
    {
    'broker_id': 81,
    'name': 'Tomar Inversiones S.A',
    'page': 'clientes2.tminversiones.com.ar'
    },
    {
    'broker_id': 88,
    'name': 'Bell Investments S.A.',
    'page': 'operar.bellbursatil.com'
    },
    {
    'broker_id': 127,
    'name': 'Maestro y Huerres S.A',
    'page': 'operar.maestroyhuerres.com'
    },
    {
    'broker_id': 153,
    'name': 'Bolsa de Comercio del Chaco',
    'page': 'clientes.bcch.org.ar'
    },
    {
    'broker_id': 163,
    'name': 'Prosecurities S.A.',
    'page': 'http://operar.psec.com.ar'
    },
    {
    'broker_id': 186,
    'name': 'Servente y Cia. S.A.',
    'page': 'http://clientes.serventeycia.com'
    },
    {
    'broker_id': 201,
    'name': 'Alfy Inversiones S.A.',
    'page': 'acceso.alfyinversiones.com.ar'
    },
    {
    'broker_id': 203,
    'name': 'Invertir en Bolsa S.A.',
    'page': 'clientesv2.invertirenbolsa.com.ar'
    },
    {
    'broker_id': 209,
    'name': 'Futuro Bursátil S.A.',
    'page': 'homebroker.futurobursatil.com.ar'
    },
    {
    'broker_id': 233,
    'name': 'Sailing S.A.',
    'page': 'login.sailinginversiones.com'
    },
    {
    'broker_id': 265,
    'name': 'Negocios Financieros y Bursátiles S.A. (Cocos Capital)',
    'page': 'cocoscap.com'
    }
]

